<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 1.30.2.19.2.46 $
 * $Author: tora60 $
 * $Date: 2012-05-04 14:07:06 $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * load_class()
 *
 * @param mixed $class
 * @param mixed $parameters
 * @return
 */
function &load_class($class, $parameters = null)
{
    require_once(PMX_SYSTEM_DIR . '/classes/' . $class . '.php');

    $name = 'pmx' . $class;

    switch (true) {
        case $parameters === false:
            /* wenn $parameters false ist, wird kein Objekt zurckgegeben */
            $return = true; // Variable wegen: Only variable references should be returned by reference
            return $return;

        case $parameters === null:
            $return = new $name();
            return $return;

        case func_num_args() > 2:
            $parameters = func_get_args();
            array_shift($parameters);

        default:
            $return = new $name($parameters);
            return $return;
    }
}

/**
 * pmx_run_hook()
 *
 * @param string $hook_name
 * @param mixed $hook_parameters
 * @param mixed $hook_result
 * @return bolean
 */
function pmx_run_hook($hook_name, &$hook_parameters = null, &$hook_result = null)
{
    if (preg_match('#[^a-z0-9.-_]#', $hook_name)) {
        return false;
    }

    /* einige Hooks verwenden diesen Variablennamen als Rckgabe.. */
    $item = &$hook_result;

    $tmp = explode('.', $hook_name);
    switch ($tmp[0]) {
        case 'user':
            $uid = intval($hook_parameters);
            $userinfo = mxGetUserDataFromUid($uid);
            break;
        case 'groups':
            $gid = intval($hook_parameters);
            break;
    }

    /* alle hook-Dateien einlesen */
    $cache = load_class('Cache');
    if (($files = $cache->read('allpmxhooks')) === false) {
        $files = array();
        foreach ((array)glob(PMX_MODULES_DIR . DS . '*' . DS . 'core' . DS . '*.php', GLOB_NOSORT) as $filename) {
            if ($filename) {
                $files[pathinfo($filename, PATHINFO_FILENAME)][] = $filename;
            }
        }
        $cache->write($files, 'allpmxhooks');
    }

    if (!isset($files[$hook_name])) {
        return false;
    }

    foreach ($files[$hook_name] as $filename) {
        include_once($filename);
    }

    return true;
}

/**
 * mxCounter()
 *
 * @return
 */
function mxCounter()
{
    global $prefix;

    if (MX_MODULE == 'admin' || MX_IS_ADMIN) {
        return true;
    }

    /* Get the Browser data */
    $browser = load_class('Browser');

    $where[] = "(`type`='total' AND `var`='hits')";
    switch (true) {
        case $browser->browser:
            /* ein Browser */
            $where[] = "(`var`='$browser->browser' AND `type`='browser')";
            $isbot = false;
            break;
        case $browser->robot:
            /* ein Bot */
            $where[] = "(`var`='$browser->robot' AND `type`='robot')";
            $isbot = true;
            break;
        default:
            $isbot = false;
    }
    /* das Betriebssystem */
    $where[] = "(`var`='$browser->os' AND `type`='os')";
    $where = implode(" OR ", $where);

    $qry = "UPDATE `${prefix}_counter` SET `count`=count+1 WHERE " . $where;
    $result = sql_system_query($qry);

    /* es mssen genau 3 Datenstze gendert werden !! */
    $affected = sql_affected_rows();
    if ($affected != 3) {
        $types = array('total' => false, 'browser' => false, 'robot' => false, 'os' => false);
        $qry = "SELECT `type` FROM `${prefix}_counter` WHERE " . $where . "";
        $result = sql_system_query($qry);
        while (list($type) = sql_fetch_row($result)) {
            if (isset($types[$type])) {
                $types[$type] = true;
            }
        }

        $vals = array();
        /* fehlende anfgen */
        if (!$types['total']) {
            $vals[] = "('total', 'hits', NULL, 1)";
        }
        if (!$isbot && !$types['browser']) {
            $vals[] = "('browser', '$browser->browser', '$browser->browser_icon', 1)";
        } else if ($isbot && !$types['robot']) {
            $vals[] = "('robot', '$browser->robot', '$browser->robot_icon', 1)";
        }
        if ($browser->os && !$types['os']) {
            $vals[] = "('os', '$browser->os', '$browser->os_icon', 1)";
        }
        if ($vals) {
            $qry = "REPLACE INTO `${prefix}_counter` (`type`, `var`, `icon`, `count`) VALUES " . implode(',', $vals);
            $result = sql_system_query($qry);
        }
    }

    $now = getdate();
    sql_system_query("UPDATE `${prefix}_stats` SET `hits`=hits+1
              WHERE (`year`='$now[year]')
                AND (`month`='$now[mon]')
                AND (`date`='$now[mday]')
                AND (`hour`='$now[hours]')");

    $affected = sql_affected_rows();
    if (!$affected) {
        $vals = array();
        for ($i = 0; $i <= 23; $i++) {
            $cnt = intval($i == $now['hours']);
            $vals[$i] = " ($now[year], $now[mon], $now[mday] ,$i, $cnt)";
        }
        $qry = "REPLACE INTO `${prefix}_stats` (`year`, `month`, `date`, `hour`,`hits`) VALUES " . implode(',', $vals);
        $result = sql_system_query($qry);
    }
}

/**
 * online()
 * Online aktualisieren
 *
 * @return
 */
function online()
{
    global $prefix, $user_prefix;
    static $done;
    if (isset($done)) {
        return true;
    }
    $done = true;
    $past = time() - MX_SETINACTIVE_MINS;
    $lasttime = mxSessionGetVar('lasttime');
    $lastday = (time() - 86400);
    $url = basename($_SERVER['REQUEST_URI']);

    if ($lasttime < $lastday && $lasttime > 0) {
        sql_system_query("DELETE FROM " . $prefix . "_visitors
            WHERE (time < " . ($lastday * 10) . " AND uid <> 0)
            OR (time < " . ($lastday) . " AND uid=0)
            OR (time > " . (time() + 60) . ")
            ");
    }

    if (defined('mxModFileLoaded') && defined('MX_MODULE')) {
        if (mxModFileLoaded == 2) {
            // die index.php geladen
            $module = 'Home';
            $url = 'index.php';
        } else {
            // die modules.php geladen
            $module = MX_MODULE;
        }
    } else {
        $module = $url;
        if (defined('mxAdminFileLoaded')) {
            // die admin.php geladen
            $module = 'Admin';
        } else if (preg_match('#^(.+)\.php#i', $url, $matches)) {
            $module = $matches[1] . '.php';
        }
        if (empty($module) || $module == 'index' || $url == '/') {
            $module = 'Home';
            $url = 'index.php';
        }
    }
    // falls gerade ein Formular gesendet wurde und evtl. nicht alle Parameter in der url stehen..
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        if (defined('mxModFileLoaded') && defined('MX_MODULE')) {
            $url = 'modules.php?name=' . MX_MODULE;
        }
    }

    if ($lasttime < $past || mxSessionGetVar("lastmodu") != $module) {
        mxSessionSetVar("lasttime", time());
        mxSessionSetVar("lastmodu", $module);
        $url = mxAddSlashesForSQL(mx_urltohtml($url));
        $url = substr($url, 0, 255);
        if (MX_IS_USER) {
            $usersession = mxGetUserSession();
            $uid = intval($usersession[0]);
            $qry = "UPDATE {$user_prefix}_users set user_lastvisit=" . time() . ", user_lastmod='" . $module . "', user_lasturl='" . $url . "', user_lastip='" . MX_REMOTE_ADDR . "' where uid=" . intval($uid);
            $result = sql_system_query($qry);
        } else {
            $uid = 0;
        }
        // visitor-tabelle aktualisieren
        sql_system_query("UPDATE ${prefix}_visitors set uid=" . intval($uid) . ", time=" . time() . ", module='" . $module . "', url='" . $url . "', ip='" . MX_REMOTE_ADDR . "' where ip='" . MX_REMOTE_ADDR . "'");
        if (!MX_IS_USER && !sql_affected_rows()) {
            sql_system_query("REPLACE INTO ${prefix}_visitors (time,ip,module,url,uid) VALUES (" . time() . ",'" . MX_REMOTE_ADDR . "','" . $module . "','" . $url . "'," . intval($uid) . ")");
        }
    }
}

/**
 * mxReferer()
 * HTTP-Referer Funktion
 *
 * @return
 */
function mxReferer()
{
    global $prefix;
    if (empty($GLOBALS['httprefmax'])) {
        // referer abgeschaltet
        return;
    }
    if (empty($_SERVER['HTTP_REFERER'])) {
        // kein referer
        return;
    }
    $ref = parse_url($_SERVER['HTTP_REFERER']);
    if (empty($ref['host']) || empty($ref['scheme'])) {
        // gefakter referer
        return;
    }
    if (empty($ref['path'])) {
        $ref['path'] = '';
    } else {
        // nur den Pfad der URL extrahieren und gleich den letzten slash entfernen, Ergebnis in $x[1]
        preg_match('#(.*)/[^/]*#', $ref['path'], $x);
        if (isset($x[1])) {
            $ref['path'] = $x[1];
        }
    }

    $self = parse_url(PMX_HOME_URL . '/index.php');

    if (empty($self['path'])) {
        $self['path'] = '';
    } else {
        preg_match('#(.*)/[^/]*#', $self['path'], $x);
        if (isset($x[1])) {
            $self['path'] = $x[1];
        }
    }
    // referer von eigener Seite
    if (preg_replace('#^www\.#i', '', $ref['host']) == preg_replace('#^www\.#i', '', $self['host'])) {
        /**
         * Pfade noch ueberpruefen
         * - wenn Serverpfad leer
         * - wenn Refererpfad leer
         * - wenn die Pfade gleich sind
         * - wenn Serverpfad in Refererpfad am Anfang enthalten ist
         * - wenn Refererpfad in Serverpfad am Anfang enthalten ist
         * dann diesen Referer nicht notieren
         */
        switch (true) {
            case !$self['path']:
            case !$ref['path']:
            case $ref['path'] === $self['path']:
            case strpos($ref['path'], $self['path']) === 0:
            case strpos($self['path'], $ref['path']) === 0:
                return;
        }
    }
    // referer die unerwuenscht sind, ignorieren
    if (@file_exists('admin/.ignore_referer')) {
        $ignore = file_get_contents('admin/.ignore_referer');
        $ignore = preg_split('#\s*,\s*#m', trim($ignore));
        if (in_array($_SERVER['HTTP_REFERER'], $ignore) || in_array($ref['host'], $ignore)) {
            return;
        }
    }

    $result = sql_system_query("select min(rid), avg(rid), count(rid) from " . $prefix . "_referer");
    list ($min, $avg, $count) = sql_fetch_row($result);

    if ($count == $GLOBALS['httprefmax']) {
        sql_system_query("DELETE FROM " . $prefix . "_referer WHERE rid = " . $min);
    } else if ($count > $GLOBALS['httprefmax']) {
        sql_system_query("DELETE FROM " . $prefix . "_referer WHERE rid < " . $avg);
    }
    pmxDebug::pause();
    sql_query("REPLACE INTO " . $prefix . "_referer (rid, url) values (" . intval(MX_TIME) . ",'" . mxAddSlashesForSQL($_SERVER['HTTP_REFERER']) . "')");
    pmxDebug::restore();
}

/**
 * mxSecureLog()
 *
 * @param mixed $log_eventid
 * @param mixed $log_event
 * @param integer $withdata
 * @return
 */
function mxSecureLog($log_eventid, $log_event, $withdata = 0)
{
    if (empty($GLOBALS["vkpsec_logging"]))
        return;
    global $prefix;
    $aid = "";
    $uname = "";
    if (MX_IS_USER) {
        $usersession = mxGetUserSession();
        @ $uname = $usersession[1];
    }
    if (MX_IS_ADMIN) {
        extract(mxGetAdminSession()); ;
    }
    $data = (empty($withdata)) ? '' : serialize($_REQUEST);
    $qry = "INSERT INTO ${prefix}_securelog
        (log_ip, log_time, log_eventid, log_event, uname, aid, request) VALUES
        ('" . MX_REMOTE_ADDR . "', " . time() . ", '" . mxAddSlashesForSQL($log_eventid) . "', '" . mxAddSlashesForSQL($log_event) . "', '" . mxAddSlashesForSQL($uname) . "', '" . mxAddSlashesForSQL($aid) . "', '" . mxAddSlashesForSQL($data) . "')";
    sql_system_query($qry);
}

/**
 * mxSessionSafeCookie()
 * sets a cookie to test the session-ID
 *
 * @param mixed $CookieName
 * @param mixed $inout
 * @return
 */
function mxSessionSafeCookie($CookieName, $inout)
{
    $type = ($CookieName == MX_SAFECOOKIE_NAME_ADMIN) ? 2 : 1;
    if ($GLOBALS["vkpSafeCookie" . $type]) {
        mt_srand((double) microtime() * 1000000);
        $checkId = mt_rand();
        $check = ($inout == 1) ? md5($checkId) : "";
        $CookieTime = ($inout == 1) ? 1 : -1;
        mxSetCookie($CookieName, $check, $CookieTime);
        mxSessionSetVar("ck" . $type, $checkId);
        $_COOKIE[$CookieName] = $check;
    }
    return true;
}

/**
 * mxGetAllBlocks()
 *
 * @param mixed $side
 * @return
 */
function mxGetAllBlocks($side = false)
{
    global $prefix;
    static $allblocks; // statisches Array
    if (!isset($allblocks)) { // wenn statisches Array noch nicht initialisiert
        $allblocks = array(); // statisches Array initialisieren
        // Spracheinstellung fuer qry
        $querylang = ($GLOBALS["multilingual"]) ? "(blanguage='" . mxAddSlashesForSQL($GLOBALS['currentlang']) . "' OR blanguage='')" : "active = 1";
        if (MX_IS_USER) { // ist User ?
            $userinfo = mxGetUserData(); // gesamte Userdaten in Array lesen
            $user_acl = (int) $userinfo["user_ingroup"]; // Gruppenzugehoerigkeit auslesen
        }
        $user_acl = (empty($user_acl)) ? 1 : $user_acl; // Gruppenzugehoerigkeit nur bei angemeldeten
        // rechte Bloecke von der Abfrage ausschliessen, wenn $index = 0
        $queryindex = (empty($GLOBALS["index"])) ? 0 : 1;
        $queryindex = ($queryindex) ? '' : "AND (position <> 'r')";
        $queryhome = (!defined('MX_HOME_FILE')) ? "AND position NOT IN('c','d')" : "";
        // linke Bloecke von der Abfrage ausschliessen, wenn $hide_left = 0
        $queryindex_l = (empty($GLOBALS["hide_left"])) ? 0 : 1;
        $queryindex_l = ($queryindex_l) ? "AND (position <> 'l')" : "";
        // print ($queryindex);
        // die Mitteilungen fuer die Startseite in Array stellen
        $allmessages = (empty($queryhome)) ? vkpGetMessages() : array();
        // die (nuke) Standardblockberechtigungen feststellen und in ein array stellen
        $query_view[] = "(view = 0)"; // Alle Besucher
        // if (MX_IS_USER)  $query_view[] = "(view = 1 AND ${prefix}_groups_blocks.group_id=".$user_acl.")"; // Nur angemeldete Benutzer
        // if (MX_IS_USER)  $query_view[] = "(view = 1)"; // Nur angemeldete Benutzer
        if (MX_IS_ADMIN) {
            $query_view[] = "(view = 2)"; // Nur Administratoren
        }
        if (!MX_IS_USER) {
            $query_view[] = "(view = 3)"; // Nur Anonyme Benutzer
        }
        if (MX_IS_USER) {
            $qry = "SELECT block_id FROM ${prefix}_groups_blocks WHERE (group_id=" . intval($user_acl) . ")";
            $result = sql_system_query($qry);
            if (is_resource($result)) { // wenn erfolgreich
                while (list ($bid) = sql_fetch_row($result)) {
                    if ($bid)
                        $bids[] = $bid;
                }
            }
            sql_free_result($result);
            if (isset($bids)) {
                $query_view[] = "(view = 1 AND bid in(" . implode(",", $bids) . "))";
            }
        }
        if (MX_IS_ADMIN && mxGetAdminPref('radminsuper')) {
            $query_view[] = "(view = 4)"; // Nur System-Administratoren
        }
        // aus dem array mit Berechtigungen eine Abfragebedingung konstruieren
        $queryview = implode(" OR ", $query_view);
        // die aus den Bedingungen zusammengesetzte SQL-Abfrage
        $qry2 = "SELECT * FROM ${prefix}_blocks
                        WHERE (active=1) AND ($querylang) AND ($queryview) $queryindex $queryindex_l $queryhome
                        ORDER BY position, weight ASC";
        $result2 = sql_system_query($qry2); // sql ausfuehren
        if (is_resource($result2)) { // wenn erfolgreich
            $lastpos = ""; #$i=0;
            // Schleife, alle Blockdaten in array lesen
            while ($block = sql_fetch_assoc($result2)) {
                // den Inhalt und weitere Daten aus dem includeten blockfile bzw. cache  holen
                $block = mxGetBlockData($block);
                // wenn kein Blockinhalt, Schleife fortsetzen
                if (empty($block['content']))
                    continue;
                // wenn eine andere Position, den Zaehler der Reihenfolge zuruecksetzen
                if ($block["position"] != $lastpos) {
                    $lastpos = $block["position"];
                    $i = ($allmessages && $block["position"] == "c") ? count($allmessages) : 0;
                }
                // Arrayschluessel 'order' mit diesem Wert belegen
                $block['order'] = $i;
                // Wenn der Titel als Sprachkonstante angegeben ist, diese verwenden
                $block['title'] = mxTranslate($block['title']);
                // jetzt endlich das statische Array mit den Blockdaten fuellen
                if ($block["position"] == 'r') {
                    // bei den rechten bloecken, unterscheiden zwischen den News- und Normalen-Bloecken
                    $isstoryblock = mxIsNewsBlockfile($block['blockfile']);
                    if ((!empty($GLOBALS['story_blocks']) && $isstoryblock) || (empty($GLOBALS['story_blocks']) && !$isstoryblock)) {
                        $allblocks['r'][$i] = $block;
                    }
                } else {
                    $allblocks[$block["position"]][$i] = $block;
                }
                // den Zaehler der Reihenfolge erhoehen
                $i ++;
            }
            sql_free_result($result2);
        }
        // die Messages in die CenterBloecke einschleifen
        if (count($allmessages)) {
            // print ("<h1>Hallo</h1>");
            $allblocks["c"] = (isset($allblocks["c"])) ? array_merge($allmessages, $allblocks["c"]) : $allmessages;
        }
    }

    switch (true) {
        case $side === false && isset($allblocks):
            // keine Seite angegeben, alle Blcke zurckgeben
            return $allblocks;
            break;
        case isset($allblocks[$side]):
            // wenn Bloecke fuer diese Seite vorhanden, diese als Array zurueckgeben
            return $allblocks[$side];
            break;
        default:
            // ansonsten ein leeres Array zurueckgeben
            return array();
    }
}

/**
 * mxCacheBlocks()
 *
 * @param array $block
 * @return array $block
 */
function mxCacheBlocks($block)
{
    global $prefix;
    $lang = $GLOBALS['currentlang'];

    if (!$block['refresh']) {
        return $block;
    }

    $data = array();
    if ($block['content']) {
        pmxDebug::pause();
        $data = (array)unserialize(base64_decode($block['content']));
        pmxDebug::restore();
    }

    $past = time() - $block['refresh'];
    $must_cache = ($block['time'] < $past) || !isset($data[$lang]);
    // $must_cache = true;
    if ($must_cache) {
        $block['cacheable'] = true; // wird in mxIncludeBlock() evtl. wieder gendert
        // blockfile includen
        $block = mxIncludeBlock($block);

        if (!$block['cacheable']) {
            // wenn der Block nicht gecached werden darf oder caching nicht aktiviert
            // den Cache fuer die aktuelle Sprache loeschen, falls vorhanden
            $qry = "UPDATE ${prefix}_blocks SET content='', refresh=0, time=0 WHERE bid=" . intval($block['bid']);
            sql_system_query($qry);
            // das Blockarray mit den Variablen des includeten Blocks direkt ausgeben
            return $block;
        }

        $data[$lang] = array('title' => $block['title'], 'content' => $block['content']);

        $qry = "UPDATE ${prefix}_blocks SET content='" . base64_encode(serialize($data)) . "', time=" . time() . " WHERE bid=" . intval($block['bid']);
        $result = sql_system_query($qry);
    }

    if (isset($data[$lang])) {
        return array_merge($block, $data[$lang]);
    }

    $block['content'] = '';
    // das neu erstellte Blockarray zurueckgeben....
    return $block;
}

/**
 * mxIncludeBlock()
 *
 * @param array $block
 * @return array $block
 */
function mxIncludeBlock($block)
{
    /* globals, die im Block auf jeden Fall vorhanden sein mssen */
    global $prefix, $user_prefix;

    $file = PMX_BLOCKS_DIR . DS . $block['blockfile'];
    $exist = is_file($file);

    $block['cacheable'] = true;
    $block['settings'] = array('before' => '', 'after' => '', 'class' => '');

    switch (true) {
        case !$block['blockfile']; // falls falscher Funktionsaufruf...
            return $block;
        case !$exist && MX_IS_ADMIN:
            // wenn blockfile fehlt, bei Admin Fehlermeldung zeigen
            $block['content'] = _BLOCKPROBLEM . '<br />' . $block['blockfile'];
            return $block;
        case !$exist:
            $block['content'] = '';
            return $block;
        default:
            $content = '';
            /* eventuelle Fehl-Ausgaben abfangen und zwischenspeichern */
            ob_start();

            /* benutzerdefinierte Konfiguration auswerten */
            $inifile = pathinfo($file);
            $inifile = $inifile['dirname'] . DS . $inifile['filename'] . '.ini';
            if (is_file($inifile)) {
                $block['settings'] = array_merge($block['settings'], parse_ini_file($inifile, true));
            }

            include($file);

            /* Ausgabe zusammensetzen */
            $block['content'] = trim(ob_get_clean()) . $block['settings']['before'] . $content . $block['settings']['after'];

            /* Wenn im Blockfile der Titel angegeben war, diesen verwenden */
            if (isset($blockfiletitle)) {
                $block['title'] = $blockfiletitle;
            }

            /* die Variable $mxblockcache (aus dem blockfile) entscheidet, ob der Block gecached werden darf */
            if (isset($mxblockcache) && !$mxblockcache) {
                $block['cacheable'] = false;
            }

            return $block;
    }
}

/**
 * vkpGetMessages()
 *
 * @return
 */
function vkpGetMessages()
{
    global $prefix;
    $querylang = ($GLOBALS['multilingual'] == 1) ? " AND (mlanguage='" . mxAddSlashesForSQL($GLOBALS['currentlang']) . "' OR mlanguage='')" : "";
    $qry = "SELECT mid, title, content, `date`, expire, view
      FROM " . $prefix . "_message
        WHERE active=1
        AND content <> ''
        " . $querylang . "
        ORDER BY `date` DESC";
    $result = sql_system_query($qry);
    if (!$result) {
        return;
    }
    $i = 0;
    $allmessages = array();
    while (list ($mid, $title, $content, $mdate, $expire, $view) = sql_fetch_row($result)) {
        if (empty($expire)) {
            $remain = _UNLIMITED;
        } else {
            $etime = (($mdate + $expire) - time()) / 3600;
            $etime = (int) $etime;
            if ($etime < 1) {
                $remain = _EXPIRELESSHOUR;
            } else {
                $remain = _EXPIREIN . " $etime " . _HOURS;
            }
        }

        switch ($view) {
            case 4: // nur Admins
                $viewfor = _MVIEWADMIN;
                $showit = MX_IS_ADMIN;
                break;
            case 3: // nur anonyme
                $viewfor = _MVIEWANON;
                $showit = (!MX_IS_USER);
                break;
            case 2: // nur angemeldete User
                $viewfor = _MVIEWUSERS;
                $showit = MX_IS_USER;
                break;
            case 1: // alle
            default:
                $viewfor = _MVIEWALL;
                $showit = 1;
                break;
        }

        if ($showit) {
            if ($viewfor && MX_IS_ADMIN) {
                $content .= '<p class="align-right tiny nowrap">[ ' . $viewfor . ' - ' . $remain . ' - <a href="admin.php?op=editmsg&amp;mid=' . $mid . '">' . _EDIT . '</a> ]</p>';
            }
            $allmessages[$i]["bid"] = 'm' . $mid;
            $allmessages[$i]["title"] = $title;
            $allmessages[$i]["content"] = $content;
            $allmessages[$i]["position"] = "c";
            $allmessages[$i]["weight"] = $i;
            $allmessages[$i]["order"] = $i;
            $i ++;
        }

        if ($expire != 0) {
            $past = time() - $expire;
            if ($mdate < $past) {
                sql_system_query("UPDATE " . $prefix . "_message SET active=0 WHERE mid=" . intval($mid));
            }
        }
    }
    return $allmessages;
}

/**
 * render_blocks()
 *
 * @param mixed $block
 * @return
 */
function render_blocks($block)
{
    if (!isset($block['order'])) {
        $block['order'] = $block['weight'];
    }
    switch (true) {
        case $block['position'] == 'c' && function_exists('thememiddlebox'):
        case $block['position'] == 'd' && function_exists('thememiddlebox'):
            thememiddlebox($block['title'], $block['content'], $block);
            break;
        case $block['position'] == 'c' && function_exists('themecenterbox'):
        case $block['position'] == 'd' && function_exists('themecenterbox'):
            /* kompatibilitaet zu < nuke-themes */
            themecenterbox($block['title'], $block['content'], $block);
            break;
        case function_exists('themesidebox'):
            themesidebox($block['title'], $block['content'], $block);
            break;
        default:
            ?><h2><?php echo $block['title'] ?></h2><div class="content"><?php echo $block['content'] ?></div><br /><?php
            break;
    }
}

/**
 * blocks()
 *
 * @param mixed $side
 * @return
 */
function blocks($side)
{
    $side = strtolower($side[0]); // Uebergabeparameter in den ersten Kleinbuchstaben umwandeln
    $blocks = mxGetAllBlocks($side); // statisches Blockarray fuer diese Seite abholen
    // wenn array leer, Funktion beenden
    if (empty($blocks)) {
        return;
    }
    // in Schleife alle Bloecke fuer diese Seite ausgeben
    foreach ($blocks as $block) {
        render_blocks($block);
    }
}

/**
 * mxInfo()
 * Gibt Copyright-Informationen aus
 *
 * @return
 */
function mxInfo()
{
    require_once(PMX_SYSTEM_DIR . '/mx_credits.php');
    $out = mxcredit_getinfo();

    if (isset($_REQUEST['name']) && $_REQUEST['name'] == 'mxcredit') {
        echo implode('<br />', $out);
        return '';
    } else {
        return $out;
    }
}

/**
 * mxSystemInfo()
 *
 * @return
 */
function mxSystemInfo()
{
    if (mxDemoMode()) {
        die('sorry, not more in DemoMode');
    }
    phpinfo();
    exit;
}

/**
 * mxGetOutputHandler()
 * hier kann ein anderer OutputHandler zugefuegt werden, als Alternative zu ob_gzhandler
 *
 * @return
 */
function mxGetOutputHandler()
{
    global $mxUseGzipCompression;

    if (empty($mxUseGzipCompression)) {
        return false;
    }

    /* Kompression auf jeden Fall abschalten */
    $mxUseGzipCompression = false;

    if (headers_sent()) {
        return false;
    }

    if (!isset($_SERVER['HTTP_ACCEPT_ENCODING'])) {
        return false;
    }

    if (!extension_loaded('zlib')) {
        return false;
    }

    if (mxIniGet('zlib.output_compression')) {
        return false;
    }

    if (mxIniGet('output_handler') == 'ob_gzhandler') {
        return false;
    }

    if (false !== strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip')) {
        /* Kompression einschalten */
        $mxUseGzipCompression = true;
        /* Handler zurueckgeben */
        return 'ob_gzhandler';
    }

    return false;
}

/**
 * mxSiteService()
 * Displays SiteService and DebugService
 *
 * @return
 */
function mxSiteService()
{
    /* Display SiteService */
    if ($GLOBALS['mxSiteService'] && !empty($GLOBALS['mxSiteServiceText'])) {
        echo '<!-- START mx_site_message --><div class="message-site">', mxNL2BR($GLOBALS['mxSiteServiceText']), '</div><!-- END mx_site_message -->';
    }

    /* Display DebugService */
    $out = '';
    if (MX_IS_ADMIN && pmxDebug::is_debugmode()) {
        $out .= '<h3>' . _MSGDEBUGMODE . '</h3>';
    }
    if ($out) {
        echo '<!-- START mx_debug_message --><div class="message-debug">', $out, '</div><!-- END mx_debug_message -->';
    }
}

/**
 * pmxUserStored
 * Die Funktionen dieser Klasse bitte nicht in eigenen Projekten verwenden!!
 * Mit der nchsten pragmaMx Version wird diese Klasse wieder ersatzlos entfernt!!
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2010
 * @version $Id: mx_system.php,v 1.30.2.19.2.46 2012-05-04 14:07:06 tora60 Exp $
 * @access public
 */
class pmxUserStored {
    private static $user = array();
    private static $isuser = false;
    private static $user_session = array();
    private static $admin = array();
    private static $isadmin = false;
    private static $admin_session = array();

    /**
     * pmxUserStored::init()
     *
     * @return
     */
    public static function init()
    {
        self::_init_admin();
        self::_init_admin_session();
        self::_init_user();

        define('MX_IS_USER' , self::$isuser); // Userberechtigung  initialisieren
        define('MX_IS_ADMIN', self::$isadmin); // Adminberechtigung initialisieren
        define('MX_IS_SYSADMIN', MX_IS_ADMIN && isset(self::$admin['radminsuper']) && self::$admin['radminsuper']); // SYS-Adminberechtigung initialisieren

    }

    /**
     * pmxUserStored::current_admindata()
     *
     * @return
     */
    public static function current_admindata()
    {
        return self::$admin;
    }

    /**
     * pmxUserStored::isadmin()
     *
     * @return
     */
    public static function isadmin()
    {
        return self::$isadmin;
    }

    /**
     * pmxUserStored::current_adminsession()
     *
     * @return
     */
    public static function current_adminsession()
    {
        return self::$admin_session;
    }

    /**
     * pmxUserStored::current_userdata()
     *
     * @return
     */
    public static function current_userdata()
    {
        return self::$user;
    }

    /**
     * pmxUserStored::isuser()
     *
     * @return
     */
    public static function isuser()
    {
        return self::$isuser;
    }

    /**
     * pmxUserStored::current_usersession()
     *
     * @return
     */
    public static function current_usersession()
    {
        return self::$user_session;
    }

    /**
     * pmxUserStored::getuserdata()
     *
     * @return
     */
    public static function getuserdata($where)
    {
        global $user_prefix, $prefix;

        if (!$where || !is_string($where)) {
            $where = 'FALSE';
        }

        $result = sql_system_query("SELECT u.*, IFNULL((YEAR(CURRENT_DATE) - YEAR(u.user_bday)) - ( RIGHT(CURRENT_DATE,5) < RIGHT(u.user_bday,5)), 0) AS user_age, a.aid AS admin_name, a.radminsuper AS admin_sys
                                FROM {$user_prefix}_users AS u
                                LEFT JOIN ${prefix}_authors AS a
                                ON a.user_uid = u.uid WHERE $where");

        if ($userinfo = sql_fetch_assoc($result)) {
            if (empty($userinfo['user_ingroup'])) {
                $userconfig = load_class('Userconfig');
                $userinfo['user_ingroup'] = $userconfig->default_group;
            }
            $userinfo['groups'][] = $userinfo['user_ingroup'];
            $userinfo['groups'][] = PMX_GROUP_ID_USER;
            if ($userinfo['admin_sys']) {
                $userinfo['groups'][] = PMX_GROUP_ID_SYSADMIN;
            }
            if ($userinfo['admin_name']) {
                $userinfo['groups'][] = PMX_GROUP_ID_ADMIN;
            }

            $userinfo['active'] = ($userinfo['user_stat'] == 1);

            if (self::$user_session && self::$user_session[0] == $userinfo['uid']) {
                $userinfo['current'] = true;
                $userinfo['user_online'] = true;
            } else {
                $userinfo['current'] = false;
                $userinfo['user_online'] = (($userinfo['user_lastvisit'] >= (time() - MX_SETINACTIVE_MINS)) && ($userinfo['user_lastmod'] != 'logout'));
            }

            return $userinfo;
        }
        return false;
    }

    /**
     * pmxUserStored::_init_user()
     *
     * @return
     */
    private static function _init_user()
    {
        if (!empty($GLOBALS['vkpSafeCookie1'])) { // nur wenn Variable gesetzt
            $checkId = md5(mxSessionGetVar('ck1'));
            $cookiechk = (empty($_COOKIE[MX_SAFECOOKIE_NAME_USER])) ? '' : $_COOKIE[MX_SAFECOOKIE_NAME_USER];
            if ($cookiechk != $checkId) {
                self::$isuser = false;
                return;
            }
        }

        $sess = mxSessionGetVar('user');
        if (empty($sess)) {
            self::$isuser = false;
            return;
        }

        $sess = base64_decode($sess);
        $sess = explode(':', $sess);
        if (empty($sess[2]) || !is_numeric($sess[0])) {
            self::$isuser = false;
            return;
        }

        $data = self::getuserdata('uid=' . intval($sess[0]) . ' AND user_stat=1');

        if ($data && $data['uname'] && $data['pass'] && $data['pass'] === $sess[2]) {
            self::$isuser = true;
            $data['user_online'] = true;
            $data['current'] = true;
            self::$user = $data;
            self::$user_session = $sess;
        }

        mxSessionDelVar('vkpnewuser'); // falls Neu-Registrierung...
    }

    /**
     * pmxUserStored::_init_admin()
     *
     * @return
     */
    private static function _init_admin()
    {
        global $prefix;

        if (!empty($GLOBALS['vkpSafeCookie2'])) { // nur wenn Variable gesetzt
            $checkId = md5(mxSessionGetVar('ck2'));
            $cookiechk = (empty($_COOKIE[MX_SAFECOOKIE_NAME_ADMIN])) ? '' : $_COOKIE[MX_SAFECOOKIE_NAME_ADMIN];
            if ($cookiechk != $checkId) {
                self::$isadmin = false;
                return;
            }
        }

        $sess = mxSessionGetVar('admin');
        if (empty($sess)) {
            self::$isadmin = false;
            return;
        }
        $sess = base64_decode($sess);
        $sess = explode(':', $sess);
        if (empty($sess[1])) {
            self::$isadmin = false;
            return;
        }

        $result = sql_system_query("select * from ${prefix}_authors where aid='" . mxAddSlashesForSQL(substr($sess[0], 0, 25)) . "'");
        $data = sql_fetch_assoc($result);

        if ($data['radminsuper']) {
            foreach ($data as $key => $value) {
                if (strpos($key, 'radmin') === 0) {
                    $data[$key] = 1;
                }
            }
        }

        if ($data && $data['pwd'] && $data['pwd'] === $sess[1]) {
            self::$admin = $data;
            self::$isadmin = true;
        }
    }

    /**
     * pmxUserStored::_init_admin_session()
     *
     * @return
     */
    private static function _init_admin_session()
    {
        if (self::$isadmin) {
            self::$admin_session = array('aid' => self::$admin['aid'],
                'pwd' => self::$admin['pwd'],
                'admlanguage' => '', // admlanguage nur noch zur Kompatibilitt
                );
        } else {
            self::$admin_session = array('aid' => '',
                'pwd' => '',
                'admlanguage' => '', // admlanguage nur noch zur Kompatibilitt
                );
        }
    }
}
function ZZZZ_OK_mxCacheBlocks($block)
{
    global $prefix;
    static $stat_langfield, $stat_langfound;
    $seperator = "@#!@";

    $must_cache = false;

    /* aktuelle Sprache statisch speichern, falls ein Block die Variable $currentlang aendert */
    if (!isset($stat_langfield)) {
        // wenn multilingual abgeschaltet, das normale Feld content verwenden
        $stat_langfield = ($GLOBALS['multilingual']) ? 'cache_' . trim($GLOBALS['currentlang']) : 'content';
    }

    switch (true) {
        case !isset($block[$stat_langfield]):
            /* Cache muss aktualisiert werden */
            $must_cache = true;
            /* wenn das caching-Feld fuer diese Sprache noch nicht in der Tabelle */
            if (!isset($stat_langfound)) {
                // Das entsprechende Feld in der Tabelle erstellen
                $result = sql_system_query("ALTER TABLE `${prefix}_blocks` ADD `" . $stat_langfield . "` TEXT NOT NULL ;");
                // Die statische Variable $stat_langfound verhindert, dass die Tabelle mehrmals geaendert wird
                $stat_langfound = ($result) ? true : false;
            }
            break;
        case !empty($block[$stat_langfield]):
            /* wenn der Cache fuer die Sprache bereits vorhanden und Cache aktiviert */
            // den codierten Cachestring zerlegen
            $cachecontent = explode($seperator, $block[$stat_langfield]);
            /* feststellen ob der Cache aktualisiert werden muss */
            $must_cache = (intval($cachecontent[0]) <= time());
            if (!$must_cache && isset($cachecontent[1])) {
                $block['title'] = base64_decode($cachecontent[1]);
                $block['content'] = base64_decode($cachecontent[2]);
            }
            $stat_langfound = true;
            break;
        default:
            /* Cache muss aktualisiert werden */
            $must_cache = true;
            $stat_langfound = true;
    }

    /* wenn Cache aktualisiert werden muss und Cache aktiviert */
    if ($must_cache) {
        $block['cacheable'] = true; // wird in mxIncludeBlock() evtl. wieder gendert
        // blockfile includen
        $block = mxIncludeBlock($block);

        if (!$block['cacheable']) {
            // wenn der Block nicht gecached werden darf oder caching nicht aktiviert
            // den Cache fuer die aktuelle Sprache loeschen, falls vorhanden
            $vari = ($stat_langfound === true) ? " , `" . $stat_langfield . "`=''" : '';
            sql_system_query("UPDATE `${prefix}_blocks` SET `refresh`=0 , `time`=0 " . $vari . " WHERE bid=" . intval($block['bid']));
            // das Blockarray mit den Variablen des Blocks direkt ausgeben
            return $block;
        }

        /* Wenn das caching-Feld fuer diese Sprache vorhanden ist */
        if ($stat_langfound === true) {
            // die Variablen des blockfiles und die naechste Refreshzeit (fuer diese Sprache)
            // in dem caching-Feld speichern
            // die base64 codierung wird benoetigt, weil im Blockfile Backslashes von Javascript-Funktionen
            // enthalten sein koennen, die beim normalen Speichern und auslesen verloren gingen
            $cachecontent[0] = time() + (int) $block['refresh'];
            $cachecontent[1] = base64_encode($block['title']);
            $cachecontent[2] = base64_encode($block['content']);
            $cachecontent = implode($seperator, $cachecontent);
            $qry = "UPDATE `${prefix}_blocks` SET `" . $stat_langfield . "`='" . $cachecontent . "' WHERE bid=" . intval($block['bid']) . ";";
            sql_system_query($qry);
        }
    }
    // das neu erstellte Blockarray zurueckgeben....
    return $block;
}

?>